/*
 * Purpose: base class for all command line oriented internet protocols
 * Author:  Frank Bu
 * Created: 2002
 */

#ifndef CMDPROT_H
#define CMDPROT_H

#include <wx/wx.h>

#include "email.h"


class wxCmdlineProtocolSocketEventHandler;

/**
 * Base class for all command line oriented internet protocols.
 */
class wxCmdlineProtocol : public wxSocketClient
{
public:
    wxCmdlineProtocol();

	~wxCmdlineProtocol();

	/**
	 * Sets the SMTP server and the optional login information.
	 * No server connection will be established until 'Send' is called.
	 * \param host SMTP server.
	 * \param user User used for login, if specified.
	 * \param password Password used for login, if specified.
	 */
	void SetHost(const wxString& host, const wxString& user = "", const wxString& password = "");

	void Connect();

	// handling for wxSOCKET_INPUT

	/**
	 * Is called for every line input.
	 * \param line The server response.
	 */
	virtual void EvaluateLine(const wxString& line) = 0;

	/**
	 * Is called if successfully connected to the server.
	 * \param event The wxSOCKET_CONNECTION event.
	 */
	virtual void OnConnect(wxSocketEvent& event) = 0;

	/**
	 * This method is called if new data is available on the socket.
	 * It extracts one line, terminated by CRLF, and calls EvaluateLine.
	 * \param event The wxSOCKET_INPUT event.
	 */
	void OnInput(wxSocketEvent& event);

	/**
	 * Message handler for wxWindows socket events.
	 */
	void OnSocketEvent(wxSocketEvent& event);

	/**
	 * Writes the specified string to the socket.
	 * \param msg The string to be written.
	 */
	void Write(const wxString& msg);

private:
	wxCmdlineProtocolSocketEventHandler* m_pCmdlineProtocolSocketEventHandler;
	bool m_bConnected;
	wxString m_inputLine;
	wxString m_host;
	wxString m_user;
	wxString m_password;
};

#endif
